@@ -1,16 +1,22 @@
 import logging
 import os
+import sys
 from copy import deepcopy
 from multiprocessing import cpu_count
 from signal import signal
 from warnings import warn
 
-import pkg_resources
 from django.conf import settings
 from django.utils.translation import gettext_lazy as _
 
 from django_q.queues import Queue
 
+# The "selectable" entry points were introduced in importlib_metadata 3.6 and Python 3.10.
+if sys.version_info < (3, 10):
+    from importlib_metadata import entry_points
+else:
+    from importlib.metadata import entry_points
+
 # optional
 try:
     import psutil
@@ -27,6 +33,11 @@ try:
 except ModuleNotFoundError:
     resource = None
 
+try:
+    import setproctitle
+except ModuleNotFoundError:
+    setproctitle = None
+
 
 class Conf:
     """
@@ -34,10 +45,25 @@ class Conf:
     """
 
     try:
-        conf = settings.Q_CLUSTER
+        conf = settings.Q_CLUSTER.copy()
     except AttributeError:
         conf = {}
 
+    _Q_CLUSTER_NAME = os.getenv("Q_CLUSTER_NAME")
+    if (
+        _Q_CLUSTER_NAME
+        and _Q_CLUSTER_NAME != conf.get("name")
+        and _Q_CLUSTER_NAME != conf.get("cluster_name")
+    ):
+        conf["cluster_name"] = _Q_CLUSTER_NAME
+        alt_conf = conf.pop("ALT_CLUSTERS")
+        if isinstance(alt_conf, dict):
+            alt_conf = alt_conf.get(_Q_CLUSTER_NAME)
+            if isinstance(alt_conf, dict):
+                alt_conf.pop("name", None)
+                alt_conf.pop("cluster_name", None)
+                conf.update(alt_conf)
+
     # Redis server configuration . Follows standard redis keywords
     REDIS = conf.get("redis", {})
 
@@ -45,15 +71,6 @@ class Conf:
 
     DJANGO_REDIS = conf.get("django_redis", None)
 
-    # Disque broker
-    DISQUE_NODES = conf.get("disque_nodes", None)
-
-    # Optional Authentication
-    DISQUE_AUTH = conf.get("disque_auth", None)
-
-    # Optional Fast acknowledge
-    DISQUE_FASTACK = conf.get("disque_fastack", False)
-
     # IronMQ broker
     IRON_MQ = conf.get("iron_mq", None)
 
@@ -74,15 +91,35 @@ class Conf:
     MONGO_DB = conf.get("mongo_db", None)
 
     # Name of the cluster or site. For when you run multiple sites on one redis server
+    # It's also the `salt` for signing OrmQ, and part of the Redis stats caching key
+    # For all clusters in one site, PREFIX should be the same value to be able to decrypt payloads
     PREFIX = conf.get("name", "default")
 
+    # Support alternative cluster name to use multiple queues in one site.
+    #   cluster name and queue name are interchangeable, same thing.
+    CLUSTER_NAME = conf.get("cluster_name", PREFIX)
+
     # Log output level
     LOG_LEVEL = conf.get("log_level", "INFO")
 
-    # Maximum number of successful tasks kept in the database. 0 saves everything. -1 saves none
+    # Maximum number of successful tasks kept in the database. 0 saves everything.
+    # -1 saves none
     # Failures are always saved
     SAVE_LIMIT = conf.get("save_limit", 250)
 
+    # save-limit can be set per Task's "group" or "name" or "func"
+    SAVE_LIMIT_PER = conf.get("save_limit_per", None)
+
+    # Verify SAVE_LIMIT_PER is valid
+    if SAVE_LIMIT_PER not in ["group", "name", "func", None]:
+        warn(
+            _(
+                "SAVE_LIMIT_PER (%(option)s) is not a valid option. Options are: "
+                "'group', 'name', 'func' and None. Default is None."
+            )
+            % {"option": SAVE_LIMIT_PER}
+        )
+
     # Guard loop sleep in seconds. Should be between 0 and 60 seconds.
     GUARD_CYCLE = conf.get("guard_cycle", 0.5)
 
@@ -112,11 +149,12 @@ class Conf:
     # Sets compression of redis packages
     COMPRESSED = conf.get("compress", False)
 
-    # Number of tasks each worker can handle before it gets recycled. Useful for releasing memory
+    # Number of tasks each worker can handle before it gets recycled.
+    # Useful for releasing memory
     RECYCLE = conf.get("recycle", 500)
 
-    # The maximum resident set size in kilobytes before a worker will recycle. Useful for limiting memory usage
-    # Not available on all platforms
+    # The maximum resident set size in kilobytes before a worker will recycle.
+    # Useful for limiting memory usage. Not available on all platforms
     MAX_RSS = conf.get("max_rss", None)
 
     # Number of seconds to wait for a worker to finish.
@@ -132,12 +170,14 @@ class Conf:
     RETRY = conf.get("retry", 60)
 
     # Verify if retry and timeout settings are correct
-    if not TIMEOUT or  (TIMEOUT > RETRY):
-        warn("""Retry and timeout are misconfigured. Set retry larger than timeout, 
-        failure to do so will cause the tasks to be retriggered before completion. 
-        See https://django-q.readthedocs.io/en/latest/configure.html#retry for details.""")
-        
-    
+    if not TIMEOUT or (TIMEOUT > RETRY):
+        warn(
+            "Retry and timeout are misconfigured. Set retry larger than timeout,"
+            "failure to do so will cause the tasks to be retriggered before completion."
+            "See https://django-q2.readthedocs.io/en/master/configure.html#retry "
+            "for details."
+        )
+
     # Sets the amount of tasks the cluster will try to pop off the broker.
     # If it supports bulk gets.
     BULK = conf.get("bulk", 1)
@@ -154,12 +194,14 @@ class Conf:
     # The Django cache to use
     CACHE = conf.get("cache", "default")
 
-    # Use the cache as result backend. Can be 'True' or an integer representing the global cache timeout.
+    # Use the cache as result backend. Can be 'True' or an integer representing the
+    # global cache timeout.
     # i.e 'cached: 60' , will make all results go the cache and expire in 60 seconds.
     CACHED = conf.get("cached", False)
 
     # If set to False the scheduler won't execute tasks in the past.
-    # Instead it will run once and reschedule the next run in the future. Defaults to True.
+    # Instead it will run once and reschedule the next run in the future. Defaults to
+    # True.
     CATCH_UP = conf.get("catch_up", True)
 
     # Use the secret key for package signing
@@ -173,7 +215,7 @@ class Conf:
     ERROR_REPORTER = conf.get("error_reporter", {})
 
     # Optional attempt count. set to 0 for infinite attempts
-    MAX_ATTEMPTS = conf.get('max_attempts', 0)
+    MAX_ATTEMPTS = conf.get("max_attempts", 0)
 
     # OSX doesn't implement qsize because of missing sem_getvalue()
     try:
@@ -197,12 +239,18 @@ class Conf:
 
     # to manage workarounds during testing
     TESTING = conf.get("testing", False)
-    
+
+    # Timezone for next_run, overrules Django timezone
+    TIME_ZONE = None
+    if settings.USE_TZ:
+        TIME_ZONE = conf.get("time_zone", settings.TIME_ZONE)
+
+
 # logger
 logger = logging.getLogger("django-q")
 
 # Set up standard logging handler in case there is none
-if not logger.handlers:
+if not logger.hasHandlers():
     logger.setLevel(level=getattr(logging, Conf.LOG_LEVEL))
     logger.propagate = False
     formatter = logging.Formatter(
@@ -234,9 +282,7 @@ if Conf.ERROR_REPORTER:
         # iterate through the configured error reporters,
         # and instantiate an ErrorReporter using the provided config
         for name, conf in error_conf.items():
-            for entry in pkg_resources.iter_entry_points(
-                "djangoq.errorreporters", name
-            ):
+            for entry in entry_points(group="djangoq.errorreporters", name=name):
                 Reporter = entry.load()
                 reporters.append(Reporter(**conf))
         error_reporter = ErrorReporter(reporters)
@@ -254,5 +300,6 @@ def get_ppid():
         return psutil.Process(os.getpid()).ppid()
     else:
         raise OSError(
-            "Your OS does not support `os.getppid`. Please install `psutil` as an alternative provider."
+            "Your OS does not support `os.getppid`. Please install `psutil` as an "
+            "alternative provider."
         )
